<?php
namespace PublishPress\Capabilities;

/*
 * PublishPress Capabilities [Free]
 * 
 * Capabilities Screen UI: Custom Statuses interface
 * 
 * Note that the free version handles only the status selection capabilities. 
 * Custom status editing and deletion capability control is Pro functionality.
 * 
 */
class CustomPrivacyCapsUI {
    var $type_caps = [];

    function __construct() {
        add_filter('pp_capabilities_extra_post_capability_tabs', [$this, 'fltAddPrivacyCapsTab']);
        add_action('publishpress-caps_manager_postcaps_section', [$this, 'drawUI']);

        add_filter('publishpress_caps_manager_typecaps', [$this, 'fltTypeCaps']);
    }

    function fltTypeCaps($type_caps) {
        return array_merge($type_caps, $this->type_caps);
    }

    function fltAddPrivacyCapsTab($tabs) {
        if (class_exists('PublishPress_Statuses')) {
            $tabs['custom-privacy'] = esc_html__('Custom Visibility', 'capabilities-pro');
        }

        return $tabs;
    }

    function drawUI ($args = []) {
        global $capsman, $cme_cap_helper, $current_user, $publishpress;

        $defaults = [
            'current' => '', 
            'rcaps' => [], 
            'is_administrator' => false, 
            'pp_metagroup_caps' => [], 
            'default_caps' => [], 
            'custom_types' => [], 
            'defined' => [], 
            'unfiltered' => [], 
            'type_caps' => [],
            'active_tab_id' => '',
        ];
        
        foreach(array_keys($defaults) as $var) {
            $$var = (isset($args[$var])) ? $args[$var] : $defaults[$var];
        }

        $this->type_caps = $type_caps;

        $list_statuses = [];

        if (!Pro::customPrivacyStatusesAvailable()) {
            return;
        }

        // This function is only called if PublishPress and its Custom Statuses module are active        
        $pp_terms = get_terms('post_status', ['hide_empty' => false]);
        foreach ($pp_terms as $term) {
            if (is_object($term)) {
                $list_statuses[$term->slug] = true;
            }
        }

        if (class_exists('PublishPress_Statuses')) {
            $ordered_statuses = \PublishPress_Statuses::getCustomStatuses(['private' => true]);

            $custom_status_post_types = \PublishPress_Statuses::instance()->options->post_types;
            $custom_status_post_types = array_filter($custom_status_post_types);
        } else {
            return;
        }

        $postmeta_statuses = [];

        if (class_exists('\PublishPress\Permissions\Statuses')) {
            if ($attributes = \PublishPress\Permissions\Statuses::attributes()) {
                if (!empty($attributes->attributes['post_status'])) {
                    foreach($ordered_statuses as $status_term) {
                        if (!isset($status_term->slug)) {
                            $status_term->slug = $status_term->name;
                        }

                        $status_obj = get_post_status_object($status_term->slug);
                        if (empty($status_obj) || empty($status_obj->private)) {
                            continue;
                        }

                        if (!empty($attributes->attributes['post_status']->conditions[$status_term->slug])) {
                            $postmeta_statuses[$status_term->slug] = get_post_status_object($status_term->slug);
                        }
                    }
                }
            }
        }

        $id = 'cme-cap-type-tables-custom-privacy';
        $div_display = ($id == $active_tab_id) ? 'block' : 'none';

        echo '<div id="' . esc_attr($id) . '" style="display:' . esc_attr($div_display) . '">';

        $url = defined('PUBLISHPRESS_STATUSES') 
        ? admin_url("admin.php?page=publishpress-statuses&status_type=visibility")
        : admin_url("admin.php?page=presspermit-visibility-statuses");

        echo "<h3><a href='" . esc_url_raw($url) . "' target='_blank'>"
        . esc_html__('Custom Visibility Capabilities', 'capabilities-pro') 
        . '</a></h3>';

        echo '<table class="widefat cme-typecaps">';

        do_action('presspermit_post_filters');

        $item_type = 'post';

        $cap_property_prefixes = get_option('presspermit_custom_privacy_edit_caps') 
        ? ['set', 'read', 'edit', 'delete']
        : ['read']; 

        $cap_tips = array( 
            'set' =>            __( 'Can assign this status (for selected post types)', 'capabilities-pro' ),
            'read' =>            __( 'Can read this status (for selected post types)', 'capabilities-pro' ),
        );

        foreach($ordered_statuses as $status_term) {
            $status = $status_term->slug;
            if (!$status_obj = get_post_status_object($status)) {
                continue;
            }

            if (in_array($status, ['private', 'future'])) {
                continue;
            }

            // If PostMeta permissions are enforced, post types that have distinct permissions, support custom statuses and don't have this status disabled
            $status_post_types = [];

            // Only control type-specific caps for set / edit / delete if they will be enforced
            if (!empty($postmeta_statuses[$status])) {
                foreach( $defined['type'] as $post_type => $type_obj ) {
                    
                    // Does this post type require distinct type-specific capabilities?
                    if ( in_array( $post_type, $unfiltered['type'] ) ) {
                        continue;
                    }

                    if (function_exists('presspermit') && !in_array($post_type, presspermit()->getEnabledPostTypes())) {
                        continue;
                    }

                    // Do PublishPress module settings enable custom statuses for this post type?
                    if (empty($custom_status_post_types[$post_type])) {
                        continue;
                    }
                    
                    if (!Pro::customStatusPostMetaPermissions($post_type, $status)) {
                        continue;
                    }

                    $status_post_types[$post_type] = $type_obj;
                }
            }

            // column header (Set / Edit / Delete)
            if (count($cap_property_prefixes) > 1 || empty($first_row_done)) {
                $tr_class = 'pp-capabilities-status-header';
                
                $status_header = "<tr class='" . esc_attr($tr_class) . "'><th></th>";

                // label cap properties
                foreach( $cap_property_prefixes as $prefix ) {
                    if (($prefix != 'set') && empty($postmeta_statuses[$status])) {
                        $status_header .= '<th></th>';
                        continue;
                    }

                    $tip = ( isset( $cap_tips[$prefix] ) ) ? $cap_tips[$prefix] : '';

                    $status_header .= "<th title='" . esc_attr($tip) . "' class='post-cap'>";
                    $status_header .= '<a href="#toggle_status_meta">' . ucwords(str_replace( '_', '<br />', $prefix )) . '</a>';
                    $status_header .= '</th>';
                }
            
                $status_header .= '</tr>';
            } else {
                $status_header = '<tr class="pp-capabilities-status-spacer"></tr>';
            }

            $first_row_done = true;

            $status_ui = "<tr class='cme_status cme-postmeta-status cme_status_{$status}'>";

            $status_ui .= "<td class='status-label status-label-advanced'><a href='#toggle_status_caps'>" . esc_html($status_obj->label) . "</a></td>";

            $first_property = true;

            foreach( $cap_property_prefixes as $prefix ) {
                $prop = "{$prefix}_{$status}_posts";

                if (!$first_property && !$any_property_caps) {
                    $status_col_ui = "<td class='status-caps'></td><td class='status-caps status-caps-" . esc_attr($prefix) . "'>";
                } else {
                    $status_col_ui = "<td class='status-caps status-caps-" . esc_attr($prefix) . "'>";
                }

                $first_property = false;
                $any_property_caps = false;

                $status_col_ui .= '<table>';
                $displayed_status_col = false;
                $displayed_types_link = false;

                foreach($defined['type'] as $post_type => $type_obj) {
                    $display_row = false;
                    
                    $td_classes = [];
                    $checkbox = '';
                    $title_text = '';

                    $row = '<tr>';

                    if (empty($custom_status_post_types[$post_type])) {
                        continue;
                    }

                    if ($prefix == 'set') {
                        if (empty($type_obj->cap->set_posts_status)) {
                            continue;
                        }

                        $cap = $type_obj->cap->set_posts_status;
                    
                    } else {
                        if (in_array($prefix, ['edit', 'delete'])) {
                            $basic_type_property = "{$prefix}_posts";

                            if (empty($type_obj->cap->$basic_type_property)) {
                                continue;
                            }
                        }

                        $cap = "{$prefix}_post";
                    }

                    $caps = Pro::getStatusCaps($cap, $post_type, $status);

                    $caps = array_diff($caps, [$cap]);
                    $cap_name = reset($caps);

                    if ($cap_name) {
                        if (in_array(
                            $cap_name, 
                            [   "status_change_{$status}", 
                                $type_obj->cap->read_private_posts, 
                                $type_obj->cap->edit_private_posts, 
                                $type_obj->cap->delete_private_posts
                            ]
                        )) {
                            continue;
                        }

                        $td_classes []= "post-cap";
                        
                        if (!empty($pp_metagroup_caps[$cap_name])) {
                            $td_classes []='cm-has-via-pp';
                        }
                        
                        if ($is_administrator || current_user_can($cap_name)) {
                            if (!empty($pp_metagroup_caps[$cap_name])) {
                                $title_text = sprintf( __( '%s: assigned by Permission Group', 'capabilities-pro' ), $cap_name );
                            } else {
                                $title_text = $cap_name;
                            }
                            
                            $disabled = '';
                            $checked = checked(1, ! empty($rcaps[$cap_name]), false );

                            $checkbox = '<label for="caps_' . $cap_name . '"><input type="checkbox" title="' . esc_attr($title_text) . '" name="caps[' . esc_attr($cap_name) . ']" id="caps_' . $cap_name . '" autocomplete="off" value="1" ' . $checked . ' ' . $disabled . ' /> ' 
                            . esc_html($type_obj->label)
                            . '</label>';

                            $this->type_caps [$cap_name] = true;
                            $display_row = true;
                            $any_property_caps = true;
                        }
                    }
                    
                    if (isset($rcaps[$cap_name]) && empty($rcaps[$cap_name])) {
                        $td_classes []= "cap-neg";
                    }
                    
                    $td_class = ( $td_classes ) ? implode(' ', $td_classes): '';
                    
                    $row .= "<td class='" . esc_attr($td_class) . " title='" . esc_attr($title_text) . ">$checkbox";

                    if ( false !== strpos( $td_class, 'cap-neg' ) )
                        $row .= '<input type="hidden" class="cme-negation-input" name="caps[' . $cap_name . ']" value="" />';

                    $row .= '</td>';
                    $row .= '</tr>';

                    if ( $display_row ) {
                        // Escaped piecemeal upstream; cannot be late-escaped until UI construction logic is reworked
                        echo $status_header . $status_ui . $status_col_ui;
                        $status_header = $status_ui = $status_col_ui = '';

                        // Escaped piecemeal upstream; cannot be late-escaped until UI construction logic is reworked
                        echo $row;
                        $displayed_status_col = true;
                    }
                }

                if ($displayed_status_col) {
                    echo '</table>';
                }


                if (!$status_ui) {
                    if (!empty($display_row)) {
                        echo '<div class="row-spacer">';
                    }

                    echo '</td>';
                }
            } // endforeach cap properties

            // status row
            if (!empty($display_status)) {
                if ($status_ui) {
                    if (!$empty($postmeta_statuses)):?>
                        <tr><th class="row-spacer"></th><th></th><th></th><th></th></tr>
                    <?php endif;

                    // Escaped piecemeal upstream; cannot be late-escaped until UI construction logic is reworked
                    echo $status_ui;
                }

                if (!empty($postmeta_statuses)) :?>
     
                <?php endif;

                echo '</tr>';
            }
            
        } // endforeach statuses
        ?>

        <script type="text/javascript">
        /* <![CDATA[ */
        jQuery(document).ready( function($) {
            $('input.cme_status_set_basic').click( function() {
                $(this).next('table').find('td.post-cap input').attr('disabled', !$(this).prop('checked'));
            });
        });
        /* ]]> */
        </script>

        </tr>

        </table>

        <?php 
        // clicking on post type name toggles corresponding checkbox selections
        ?>
        <script type="text/javascript">
        /* <![CDATA[ */
        jQuery(document).ready( function($) {
            $('a[href="#toggle_status_caps"]').click( function() {
                var chks = $(this).closest('tr').find('input');

                $(chks).attr( 'checked', !$(chks).first().is(':checked') );
                return false;
            });

            $('a[href="#toggle_status_meta"]').click( function() {
                var tdIndex = $(this).closest('th').index() - 1;
                var chks = $(this).closest('tr').next().find("td.status-caps:eq(" + tdIndex + ") table tr td.post-cap input");
                $(chks).attr( 'checked', ! $(chks).first().prop('checked') );
                return false;
            });

            $('div.cme-show-status-hint').click(function() {
                $(this).hide();
                $('div.pp-status-control-notice').show();
                return false;
            });
        });
        /* ]]> */
        </script>

        </div>

        <?php
    }
}
