<?php
namespace PublishPress\StatusCapabilities;

class StatusAdmin
{
    public $type_caps;
    public $postmeta_caps_displayed;

    public function __construct() {
        add_action('presspermit_statuses_edit_status_tab', [$this, 'act_status_edit_tab'], 10, 3);
    }

    public function act_status_edit_tab($tab, $status, $default_tab) {
        if ('post_access' != $tab) {
            return;
        }

        $display = ($default_tab == $tab) ? '' : 'display:none';
        
        $table_class = 'form-table pp-statuses-options';
        $attrib_type = 'moderation';
        $new = false;

        $status_obj = $status;
        $status = $status_obj->slug;

        $status_label = (!empty($status_obj->label)) ? $status_obj->label : '';
        $status_types = (!empty($status_obj) && !empty($status_obj->post_type)) ? $status_obj->post_type : [];

        $name_disabled = ($new) ? '' : ' disabled ';
        $label_disabled = ('future' == $status) ? ' disabled ' : '';

        $is_pubpress_status = !empty($status_obj->pp_custom);
        $pubpress_disable_edit = ($is_pubpress_status) ? ' disabled ' : '';

        if ('moderation' == $attrib_type) {
            $ordered_workflow_statuses = \PublishPress_Statuses::orderStatuses(
                \PublishPress\StatusCapabilities::getPostStatuses(['moderation' => true], 'object')
            );

            $capability_status = (!empty($status_obj) && !empty($status_obj->capability_status)) ? $status_obj->capability_status : '';

            $status_order = (!empty($status_obj) && isset($status_obj->order)) ? $status_obj->order : '';
            $status_parent = (!empty($status_obj) && isset($status_obj->status_parent)) ? $status_obj->status_parent : '';
        }

        ?>
        <div id='pp-<?php echo esc_attr($tab);?>' style='clear:both;margin:0;<?php echo esc_attr($display)?>' class='pp-options'>
        
        <?php
        do_action("presspermit_" . esc_attr($tab) . "_options_pre_ui");
        ?>
        
        <table class='<?php echo esc_attr($table_class);?>' id='pp-<?php echo esc_attr($tab);?>_table' style='<?php echo esc_attr($display);?>'>

        <?php
        if (empty($status_obj->publish) && empty($status_obj->private) && !in_array($status, ['future', 'draft'])) : ?>
            <tr>
            <th><label for="status_capability_status"><?php esc_html_e('Capability Requirements', 'publishpress-status-capabilities') ?></label></th>

            <td>
                <select name="status_capability_status" id="status_capability_status" autocomplete="off">
                    <option value="" <?php if ('' == $capability_status) echo ' selected '; ?>><?php esc_html_e('(Default Capabilities)', 'publishpress-status-capabilities'); ?></option>
                    
                    <option value="<?php echo esc_attr($status); ?>" <?php if ($status == $capability_status) echo ' selected '; ?>>
                    <?php esc_html_e('Custom Capabilities for this Status', 'publishpress-status-capabilities'); ?>
                    </option>
                    
                    <?php
                    foreach ($ordered_workflow_statuses as $other_status => $other_status_obj) :
                        if ($status == $other_status) {
                            continue;
                        }

                        if (!empty($other_status_obj->status_parent) && ($other_status_obj->status_parent != $status_parent)) {
                            continue;
                        }
                        
                        $selected = ($other_status == $capability_status) ? ' selected ' : '';

                        if (($other_status == $status_parent)
                        || (\PublishPress\StatusCapabilities::postStatusHasCustomCaps($other_status)
                            && (!isset($other_status_obj->capability_status) || ($other_status_obj->capability_status == $other_status))
                            )
                        ) :?>
                            <option value="<?php echo esc_attr($other_status); ?>" <?php echo esc_attr($selected); ?>>
                            <?php 
                            if ($other_status == $status_parent) {
                                printf(esc_html__('same as PARENT STATUS (%s)', 'publishpress-status-capabilities'), esc_html($other_status_obj->label));

                            } else {
                                printf(esc_html__('same as %s status', 'publishpress-status-capabilities'), esc_html($other_status_obj->label));
                            }
                            ?>
                            </option>
                        <?php endif;
                    endforeach;
                    ?>
                </select>

                <div class="pp-subtext"><?php esc_html_e('Enforce status-specific capabilities, or share capabilities with another status.', 'publishpress-status-capabilities'); ?></div>
            </td>
            </tr>
        <?php endif;

        if (!empty($status_obj->public) || !empty($status_obj->private) || in_array($status_obj->name, ['draft', 'future'])) {
            $do_postmeta_permissions = false;
        } else {
            $do_postmeta_permissions = \PublishPress\StatusCapabilities::customStatusPostMetaPermissions('', $status_obj);
        }

        if (empty($capability_status) || ($status_obj->name != $capability_status)) {
            do_action("presspermit_" . esc_attr($tab) . "_options_ui");
            ?>

            </table></div>

            <?php
            return;
        }

        $is_administrator = (function_exists('presspermit')) ? presspermit()->isContentAdministrator() : is_super_admin();

        do_action('presspermit_post_filters');
        $status_cap_mapper = \PublishPress\StatusCapabilities\CapabilityFilters::instance();

        $item_type = 'post';

        $cap_property_prefixes = ['set', 'edit', 'edit_others', 'delete', 'delete_others'];

        $cap_tips = array( 
            'set' =>            esc_html__( 'Can assign this status (for selected post types)', 'capabilities-pro' ),
            'edit' =>           esc_html__( 'Can edit posts of this status (for selected post types)', 'capabilities-pro' ),
            'edit_others' =>    esc_html__( 'Can edit other\'s posts of this status (for selected post types)', 'capabilities-pro' ),
            'delete' =>         esc_html__( 'Can delete posts of this status (for selected post types)', 'capabilities-pro' ),
            'delete_others' =>  esc_html__( 'Can delete other\'s posts of this status (for selected post types)', 'capabilities-pro' ),
        );

        $status_name = $status_obj->name;

        // Post types that have custom statuses enabled at all
        $custom_status_post_types = \PublishPress_Statuses::instance()->options->post_types;
        $custom_status_post_types = array_filter($custom_status_post_types);

        // Find post types that have distinct permissions, support custom statuses and don't have this status disabled
        $status_post_types = [];

        // Only control type-specific caps for set / edit / delete if they will be enforced
        foreach(array_keys($custom_status_post_types) as $post_type) {
            if (!in_array($post_type, array_keys(\PublishPress\StatusCapabilities::getEnabledPostTypes()))) {
                continue;
            }

            $status_post_types[$post_type] = get_post_type_object($post_type);
        }
        ?>

        <tr>
        <th colspan=2><label for="status_capabilities"><?php esc_html_e('Role Capabilities', 'publishpress-status-capabilities') ?></label>

        <span style="float: right">
        <?php
            submit_button(esc_html__('Update Status', 'publishpress'), 'primary', 'submit', false); 
        ?>
        </span>
        <br />

        <script type="text/javascript">
        /* <![CDATA[ */
        jQuery(document).ready(function ($) {
            $('input[name*="roles_set_status"]').on('click', function()
            {
                $('input[name="' + $(this).attr('name') + '"]').prop('checked', $(this).prop('checked'));
            });

            $('input.cme_status_set_basic').click( function() {
                $(this).next('table').find('td.post-cap input').attr('disabled', !$(this).prop('checked'));
            });

            $('a[href="#toggle_status_caps"]').click( function() {
                var chks = $(this).closest('tr').find('input');
                $(chks).prop( 'checked', ! $(chks).first().is(':checked') );
                return false;
            });
        });
        /* ]]> */
        </script>

        <style>
        td.post-cap {
            padding: 4px !important;
        }

        td.cm-has-via-pp {
            background-color: #84fb84;
        }
        </style>

        <br /><br />

        <table class="widefat cme-typecaps">

        <tr class=''>
            <th class='pp-caps-status-label'>
            <?php printf(esc_html__('"%s" status:', 'publishpress_statuses'), esc_html($status_obj->label));?>
            </th>

            <?php
            // label cap property column headers
            foreach( $cap_property_prefixes as $prefix ) :
                $tip = ( isset( $cap_tips[$prefix] ) ) ? $cap_tips[$prefix] : '';
            ?>
                <th title='<?php echo esc_attr($tip);?>' class='post-cap'>
                <?php echo ucwords(str_replace( '_', '<br />', $prefix ));  // phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
                ?>
                </th>
            <?php endforeach; ?>
        </tr>

        <?php
        $roles = \PublishPress_Functions::getRoles(true);

        foreach ($roles as $role_name => $role_label) {

            if (!\PublishPress_Functions::isEditableRole($role_name)) {
                continue;
            }

            $role = get_role($role_name);
            $rcaps = $role->capabilities;

            $pp_metagroup_caps = self::get_metagroup_caps( $role_name );
            ?>

            <tr class='cme_status cme-postmeta-status cme_status_<?php echo esc_attr($status_name);?>'>

                <td class='status-label status-label-advanced'><a href='#toggle_status_caps'><?php echo esc_html($role_label);?></a>

                <?php
                if (defined('PUBLISHPRESS_CAPS_VERSION')) :
                    $url = admin_url("admin.php?page=pp-capabilities&role={$role_name}");
                    ?>
                    <br /><span><a href='<?php echo esc_url($url);?>' target='rolecaps'><?php esc_html_e('edit');?></a></span>
                <?php endif;?>

                </td>

            <?php
            foreach ( $cap_property_prefixes as $prefix ) {
                $prop = "{$prefix}_{$status_name}_posts";
                ?>

                <td>

                <?php

                if ('set' == $prefix) {
                    $td_classes = ['post-cap', 'cme_status_set_basic'];
                    $cap_slug = str_replace('-', '_', $status_name);
                    $cap_name = "status_change_{$cap_slug}";
                    $status_change_cap = $cap_name;

                    if (!empty($pp_metagroup_caps[$cap_name])) {
                        $td_classes []= 'cm-has-via-pp';
                    }
                    
                    if ($is_administrator || current_user_can($cap_name)) {
                        if (!empty($pp_metagroup_caps[$cap_name])) {
                            $title_text = sprintf( __( '%s: assigned by Permission Group', 'capabilities-pro' ), $cap_name );
                        } else {
                            $title_text = $cap_name;
                        }
                        
                        $disabled = (('set' == $prefix) && empty($rcaps[$status_change_cap])) ? 'disabled' : '';

                        $td_class = ( $td_classes ) ? implode(' ', $td_classes) : '';
                        ?>

                        <input type="checkbox" class="<?php echo esc_attr($td_class);?>" title="<?php echo esc_attr($title_text);?>" 
                            name="roles_set_status[<?php echo esc_attr($role_name);?>]" autocomplete="off" 
                            value="1" <?php checked(1, ! empty($rcaps[$cap_name]));?> style="margin-bottom: 10px;" />
                        
                        <?php
                        $display_status = true;
        
                        // phpcs:ignore Squiz.PHP.CommentedOutCode.Found
                        //$this->type_caps[$cap_name] = true;   // @todo: filter Additional Caps / Plugin Caps
                    }

                    // @todo: filter Additional Caps / Plugin Caps ?
                    //$this->type_caps[$cap_name] = true;               // phpcs:ignore Squiz.PHP.CommentedOutCode.Found
                }

                $displayed_status_col = false;
                ?>
                <table>

                <?php
                foreach ($status_post_types as $post_type => $type_obj) {
                    $display_row = false;
                    
                    $td_classes = [];
                    $title_text = '';

                    // if edit_others is same as edit_posts cap, don't display a checkbox for it
                    if ( ($prefix != 'edit_others' || $type_obj->cap->edit_others_posts != $type_obj->cap->edit_posts)
                    && ($prefix != 'delete_others' || $type_obj->cap->delete_others_posts != $type_obj->cap->delete_posts)
                    && (($prefix != 'set') || !empty($type_obj->cap->set_posts_status))
                    ) {
                        if ($prefix == 'set') {
                            $cap = $type_obj->cap->set_posts_status;
                        } else {
                            $basic_type_property = "{$prefix}_posts";
                            $cap = (in_array($prefix, ['edit_others', 'delete_others'])) ? $type_obj->cap->$basic_type_property : "{$prefix}_post";
                        }

                        $caps = self::getStatusCaps($cap, $post_type, $status_name);

                        $caps = array_diff($caps, [$cap]);
                        $cap_name = reset($caps);

                        if ($cap_name) {
                            $td_classes []= "post-cap";
                            
                            if (!empty($pp_metagroup_caps[$cap_name])) 
                            {
                                $td_classes []='cm-has-via-pp';
                            }
                            
                            if ($is_administrator || current_user_can($cap_name)) {
                                if (!empty($pp_metagroup_caps[$cap_name])) {
                                    $title_text = sprintf( __( '%s: assigned by Permission Group', 'capabilities-pro' ), $cap_name );
                                } else {
                                    $title_text = $cap_name;
                                }
                                
                                $this->type_caps [$cap_name] = true;
                                $display_row = true;

                                $this->postmeta_caps_displayed = true;
                                $last_status_has_postmeta_caps = true;
                            }
                        }
                    
                    } elseif(!empty($type_obj->cap->$prop)) {
                        $title_text = sprintf( esc_attr__('shared capability: %s', 'capabilities-pro'), esc_attr($type_obj->cap->$prop));
                    }

                    if (isset($rcaps[$cap_name]) && empty($rcaps[$cap_name])) {
                        $td_classes []= "cap-neg";
                    }
                    
                    $td_class = ( $td_classes ) ? implode(' ', $td_classes): '';

                    if ( $display_row ) {
                    ?>
                        <?php
                            $td_classes = [];
                            $checkbox = '';
                            $title_text = '';
                            ?>

                            <tr>
                            
                            <?php
                            $display_cap = false;

                            // if edit_others is same as edit_posts cap, don't display a checkbox for it
                            if ( ($prefix != 'edit_others' || $type_obj->cap->edit_others_posts != $type_obj->cap->edit_posts)
                            && ($prefix != 'delete_others' || $type_obj->cap->delete_others_posts != $type_obj->cap->delete_posts)
                            && (($prefix != 'set') || !empty($type_obj->cap->set_posts_status))
                            ) {
                                if ($prefix == 'set') {
                                    $cap = $type_obj->cap->set_posts_status;
                                } else {
                                    $basic_type_property = "{$prefix}_posts";
                                    $cap = (in_array($prefix, ['edit_others', 'delete_others'])) ? $type_obj->cap->$basic_type_property : "{$prefix}_post";
                                }

                                $caps = self::getStatusCaps($cap, $post_type, $status_name);

                                $caps = array_diff($caps, [$cap]);
                                $cap_name = reset($caps);

                                if ($cap_name) {
                                    $td_classes []= "post-cap";
                                    
                                    if (!empty($pp_metagroup_caps[$cap_name])) 
                                    {
                                        $td_classes []='cm-has-via-pp';
                                    }
                                    
                                    if ($is_administrator || current_user_can($cap_name)) {
                                        $display_cap = true;

                                        if (!empty($pp_metagroup_caps[$cap_name])) {
                                            $title_text = sprintf( esc_html__( '%s: assigned by Permission Group', 'capabilities-pro' ), $cap_name );
                                        } else {
                                            $title_text = $cap_name;
                                        }

                                        $this->type_caps [$cap_name] = true;

                                        $this->postmeta_caps_displayed = true;
                                        $last_status_has_postmeta_caps = true;
                                    }
                                }
                            
                            } elseif (!empty($type_obj->cap->$prop)) {
                                $title_text = sprintf( esc_attr__('shared capability: %s', 'capabilities-pro'), esc_attr($type_obj->cap->$prop));
                            }

                            $td_class = ( $td_classes ) ? implode(' ', $td_classes): '';
                            ?>

                            <td class='<?php echo esc_attr($td_class);?>' title='<?php echo esc_attr($title_text);?>'>

                            <?php if ($display_cap) :?>
                                <input type="hidden" name="status_caps[<?php echo esc_attr($role_name);?>][<?php echo esc_attr($cap_name);?>]" autocomplete="off" value="0" />

                                <?php
                                $disabled = (('set' == $prefix) && empty($rcaps[$status_change_cap])) ? 'disabled' : '';
                                ?>

                                <label>
                                <input type="checkbox" title="<?php echo esc_attr($title_text);?>" 
                                name="status_caps[<?php echo esc_attr($role_name);?>][<?php echo esc_attr($cap_name);?>]" autocomplete="off" 
                                value="1" <?php checked(1, ! empty($rcaps[$cap_name]));?> <?php echo esc_attr($disabled);?> /> 
                                <?php echo esc_html($type_obj->label);?>
                                </label>
                            <?php endif;?>

                            <?php
                            if ( false !== strpos( $td_class, 'cap-neg' ) ) :?>
                                <input type="hidden" class="cme-negation-input" name="caps[<?php echo esc_attr($cap_name);?>]" value="" />
                            <?php 
                            endif;
                            ?>

                            </td>
                            </tr>
                    <?php
                    } // if display_row

                    if (!empty($status_header_done)) {
                        if (!empty($display_row)) {
                            echo '<div class="row-spacer">';
                        }
                        echo '</td>';
                    }

                } // endforeach status_post_types
                ?>

                </table>

                </td>

            <?php
            } // endforeach cap properties

            ?>
            <tr class=''>
            <th class='pp-caps-status-label'></th>
            </tr>

            <?php
        } // foreach $roles
        ?>
        </table>

        </th>
        </tr>
        <?php

        do_action("presspermit_" . esc_attr($tab) . "_options_ui");

        echo '</table></div>';
    }

    private static function get_metagroup_caps( $default ) {
        if (function_exists('presspermit')) {
            // @todo: API to Permissions Pro
            global $wpdb;

            // Direct query of plugin table for plugin admin UI generation

            // phpcs:ignore WordPress.DB.DirectDatabaseQuery.DirectQuery, WordPress.DB.DirectDatabaseQuery.NoCaching
            $pp_supplemental_roles = $wpdb->get_col( 
                $wpdb->prepare( 
                    "SELECT role_name FROM $wpdb->ppc_roles AS r INNER JOIN $wpdb->pp_groups AS g ON g.ID = r.agent_id AND r.agent_type = 'pp_group' WHERE g.metagroup_type = 'wp_role' AND g.metagroup_id = %s", 
                    $default 
                ) 
            );

            $pp_filtered_types = presspermit()->getEnabledPostTypes();
            $pp_metagroup_caps = [];
            $pp_cap_caster = presspermit()->capCaster();

            foreach( $pp_supplemental_roles as $_role_name ) {
                $role_specs = explode( ':', $_role_name );
                if ( empty($role_specs[2]) || ! in_array( $role_specs[2], $pp_filtered_types ) )
                    continue;

                // add all type-specific caps whose base property cap is included in this pattern role
                // i.e. If 'edit_posts' is in the pattern role, grant $type_obj->cap->edit_posts
                $pp_metagroup_caps = array_merge( $pp_metagroup_caps, array_fill_keys( $pp_cap_caster->get_typecast_caps( $_role_name, 'site' ), true ) );
            }
        
            return $pp_metagroup_caps;
        } else {
            return [];
        }
    }
    
    private static function getStatusCaps($cap, $post_type, $post_status) {
        if (!$attributes = \PublishPress\StatusCapabilities::instance()) {
            return [$cap];
        }

        if (!\PublishPress\StatusCapabilities::customStatusPostMetaPermissions()) {
            return [$cap];
        }

        if (!isset($attributes->attributes['post_status']->conditions[$post_status])) {
            return [$cap];
        }

        $caps = [];

        if (isset($attributes->condition_metacap_map[$post_type][$cap]['post_status'][$post_status])) {
            $caps = array_merge($caps, (array) $attributes->condition_metacap_map[$post_type][$cap]['post_status'][$post_status]);
        }

        if (!empty($attributes->condition_cap_map[$cap]['post_status'][$post_status])) {
            $caps = array_merge($caps, (array) $attributes->condition_cap_map[$cap]['post_status'][$post_status]);
        }

        return $caps;
    }

} // end class
